/*!
 * @file shader.h
 * @brief Definition of the Shader rendering class
*/

#ifndef _SHADER_H
#define _SHADER_H

#include "base_object.h"
#include "glincl.h"
#include <vector>


// FORWARD DECLARATION



//! The Shader is a Class-wrapper around the OpenGL Shader Language (GLSL).
class Shader : public BaseObject
{
  ObjectListDeclaration(Shader);
public:
  class Uniform
  {
  public:
    Uniform(const Shader* shader, const std::string& location) { this->uniform = glGetUniformLocationARB(shader->getProgram(), location.c_str()) ; }
    Uniform(const Shader& shader, const std::string& location) { this->uniform = glGetUniformLocation(shader.getProgram(), location.c_str()) ; };
    Uniform(GLhandleARB shaderProgram, const std::string& location) { this->uniform = glGetUniformLocation(shaderProgram, location.c_str()) ; };

    void set(float v0) const { glUniform1f(this->uniform, v0); }
    void set(float v0, float v1) const { glUniform2f(this->uniform, v0, v1); }
    void set(float v0, float v1, float v2) const { glUniform3f(this->uniform, v0, v1, v2); }
    void set(float v0, float v1, float v2, float v3) const { glUniform4f(this->uniform, v0, v1, v2, v3); }

    void set(int v0) const { glUniform1i(this->uniform, v0); }
    void set(int v0, int v1) const { glUniform2i(this->uniform, v0, v1); }
    void set(int v0, int v1, int v2) const { glUniform3i(this->uniform, v0, v1, v2); }
    void set(int v0, int v1, int v2, int v3) const { glUniform4i(this->uniform, v0, v1, v2, v3); }

    void setV(unsigned int count, float* vv) const;
    void setV(unsigned int count, int* vv) const;

  private:
    GLint uniform;
  };

  //! The Type of Shader.
  typedef enum
  {
    None       = 0,     //!< No Type at all
    Fragment   = 1,     //!< Fragment Shader.
    Vertex     = 2,     //!< Vertex Shader.
    Program    = 4,     //!< Compiled Shader Programm.
  }  Type;

public:
  Shader(const std::string& vertexShaderFile = "", const std::string& fragmentShaderFile = "");
  virtual ~Shader();


  Shader::Uniform getUniform(const std::string& location) { return Shader::Uniform(this, location); }

  GLhandleARB getProgram() const { return this->shaderProgram; }
  GLhandleARB getVertexS() const { return this->vertexShader; }
  GLhandleARB getFragmentS() const { return this->fragmentShader; }

  static bool checkShaderAbility();
  void activateShader();
  static void deactivateShader();

  void debug() const;

  inline static bool isShaderActive() { return (Shader::storedShader != NULL) ? true : false; };
  inline static Shader* getActiveShader() { return Shader::storedShader; };
  inline static void suspendShader() { Shader* currShader = storedShader; if (storedShader!= NULL) { Shader::deactivateShader(); Shader::storedShader = currShader;} };
  inline static void restoreShader() { if (storedShader != NULL) storedShader->activateShader(); storedShader = NULL; };

  static void printError(GLhandleARB program);

private:
  void bindShader(const char* name, const float* value, size_t size);
  void linkShaderProgram();
  bool readShader(const std::string& fileName, std::string& output);
  bool loadShaderProgramm(Shader::Type type, const std::string& fileName);
  void deleteProgram(Shader::Type type);

private:
  std::string            fragmentShaderFile;
  std::string            vertexShaderFile;

  GLhandleARB            shaderProgram;

  GLhandleARB            vertexShader;
  GLhandleARB            fragmentShader;


  static Shader*         storedShader;
};

#endif /* _SHADER_H */
